<?php

namespace TwoFAS\Light\TOTP;

use TwoFAS\Light\Exception\Invalid_TOTP_Secret;
use TwoFAS\Light\Exception\Invalid_TOTP_Token_Format;
use TwoFAS\Light\Exception\Invalid_TOTP_Token_Supplied;

class Code_Validator {
	
	/** @var Code_Generator */
	private $totp_code_generator;
	
	/** @var Format_Validator */
	private $format_validator;
	
	/**
	 * Code_Validator constructor.
	 *
	 * @param Code_Generator   $totp_code_generator
	 * @param Format_Validator $format_validator
	 */
	public function __construct( Code_Generator $totp_code_generator, Format_Validator $format_validator ) {
		$this->totp_code_generator = $totp_code_generator;
		$this->format_validator    = $format_validator;
	}
	
	/**
	 * @param string $totp_token
	 *
	 * @throws Invalid_TOTP_Secret
	 * @throws Invalid_TOTP_Token_Format
	 * @throws Invalid_TOTP_Token_Supplied
	 */
	public function validate_code( $totp_token ) {
		$this->format_validator->validate_token_format( $totp_token );
		$valid_tokens = $this->totp_code_generator->generate_tokens();
		
		if ( ! in_array( $totp_token, $valid_tokens, true ) ) {
			throw new Invalid_TOTP_Token_Supplied();
		}
	}
}
