<?php

namespace TwoFAS\Light\TOTP;

use TwoFAS\Light\Exception\Invalid_TOTP_Secret;
use TwoFAS\Light\Exception\Invalid_TOTP_Token_Format;
use TwoFAS\Light\Exception\Invalid_TOTP_Token_Supplied;
use TwoFAS\Light\Exception\Validation_Exception;
use TwoFAS\Light\App;
use TwoFAS\Light\User\User;
use TwoFAS\Light\User\User_Blocker;

class TOTP_Login_Validator {
	
	/** @var Format_Validator */
	private $format_validator;
	
	public function __construct( Format_Validator $format_validator ) {
		$this->format_validator = $format_validator;
	}
	
	/**
	 * @param App  $app
	 * @param User $authenticating_user
	 *
	 * @throws Validation_Exception
	 */
	public function validate( App $app, User $authenticating_user ) {
		$request        = $app->get_request();
		$user_blocker   = new User_Blocker( $app->get_time(), $authenticating_user );
		$code_validator = $app->get_totp_code_validator( $authenticating_user->get_totp_secret() );
		
		//  If valid step and totp token are supplied,
		$totp_token = $request->get_from_post( 'ciberprotector_totp_token' );
		
		if ( ! is_string( $totp_token ) ) {
			throw new Validation_Exception();
		}
		
		if ( empty( $totp_token ) ) {
			throw new Validation_Exception( 'Token cannot be empty' );
		}
		
		try {
			$code_validator->validate_code( $totp_token );
		} catch ( Invalid_TOTP_Token_Format $e ) {
			throw new Validation_Exception( 'Token is not in a valid format' );
		} catch ( Invalid_TOTP_Secret $e ) {
			$this->handle_invalid_token( $user_blocker );
		} catch ( Invalid_TOTP_Token_Supplied $e ) {
			$this->handle_invalid_token( $user_blocker );
		}
		
		if ( $user_blocker->is_blocked() ) {
			throw new Validation_Exception( 'Your account is temporarily blocked' );
		}
		
		$user_blocker->handle_successful_login();
	}
	
	/**
	 * @param User_Blocker $user_blocker
	 *
	 * @throws Validation_Exception
	 */
	private function handle_invalid_token( User_Blocker $user_blocker ) {
		$user_blocker->handle_failed_login();
		throw new Validation_Exception( 'Token is invalid' );
	}
}
