<?php

namespace TwoFAS\Light\View;

use Twig_Environment;
use Twig_Error_Loader;
use Twig_Error_Runtime;
use Twig_Error_Syntax;
use Twig_Loader_Filesystem;
use Twig_SimpleFunction;
use TwoFAS\Light\Browser\Browser;
use TwoFAS\Light\Time\Time;

class View_Renderer
{

	const ERROR_TEMPLATE_NOT_FOUND = 'CiberProtector plugin could not find a template.';
	const ERROR_COMPILATION_FAILED = 'Error occurred in CiberProtector plugin during template compilation.';
	const ERROR_RENDERING_FAILED = 'Error occurred in CiberProtector plugin during template rendering.';

	/**
	 * @var string
	 */
	private $templates_path;

	/**
	 * @var Twig_Environment
	 */
	private $twig;

	/** @var Time */
	private $time;

	/**
	 * View_Renderer constructor.
	 *
	 * @param Time $time
	 */
	public function __construct(Time $time)
	{
		$this->time = $time;
	}

	public function init()
	{
		$this->templates_path = __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'view';
		$twig_loader = new Twig_Loader_Filesystem($this->templates_path);
		$this->twig = new Twig_Environment($twig_loader);
		$this->twig->addFunction(new Twig_SimpleFunction(
			'timestamp_to_wp_datetime',
			array($this->time, 'format_to_string_using_wp_settings')
		));
		$this->twig->addFunction(new Twig_SimpleFunction('describe_device', array($this, 'describe_device')));
		$this->twig->addFunction(new Twig_SimpleFunction('login_footer', 'login_footer'));
		$this->twig->addFunction(new Twig_SimpleFunction('login_header', 'login_header'));
	}

	/**
	 * @param $template
	 * @param $arguments
	 *
	 * @return string
	 */
	public function render($template, $arguments)
	{
		$arguments['ciberprotector_plugin_path'] = CIBERPROTECTOR_PLUGIN_PATH;
		$arguments['ciberprotector_admin_path'] = CIBERPROTECTOR_WP_ADMIN_PATH;
		$arguments['ciberprotector_full_plugin_is_active'] = defined('CIBERPROTECTOR_FULL_TWOFAS_PLUGIN_ACTIVE_FLAG');
		$arguments['login_url'] = wp_login_url();
		$arguments['nonce_field'] = wp_nonce_field(
			'ciberprotector_ajax',
			$name = '_wpnonce',
			$referer = false,
			$echo = false
		);

		try {
			return $this->twig->render($template, $arguments);
		} catch (Twig_Error_Loader $e) {
			return $this->render_error(self::ERROR_TEMPLATE_NOT_FOUND);
		} catch (Twig_Error_Syntax $e) {
			return $this->render_error(self::ERROR_COMPILATION_FAILED);
		} catch (Twig_Error_Runtime $e) {
			return $this->render_error(self::ERROR_RENDERING_FAILED);
		}
	}

	/**
	 * @param $user_agent
	 *
	 * @return string
	 */
	public function describe_device($user_agent)
	{
		$ciberprotector_browser = new Browser($user_agent);

		return $ciberprotector_browser->describe();
	}

	/**
	 * @param string $message
	 *
	 * @return string
	 */
	private function render_error($message)
	{
		return '<div class="ciberprotector-view-renderer-error-container"><div class="notice notice-error error ciberprotector-view-renderer-error"><p>' . $message . '</p></div></div>';
	}
}
